#include <stdlib.h>
#include <stdbool.h>
#include <string.h>
#include <openssl/evp.h>

#include "aead.h"

bool aead_encrypt(unsigned char* ciphertext, unsigned char* tag, const unsigned char* plaintext, size_t ptLen, const unsigned char* key, const unsigned char* associatedData, size_t adLen, const unsigned char* nonce) {
	bool result = false;

	if (adLen > INT_MAX) return false;
	if (ptLen > INT_MAX) return false;

	EVP_CIPHER_CTX* ctx = EVP_CIPHER_CTX_new();
	if (ctx == NULL) return result;

	if (EVP_EncryptInit_ex(ctx, EVP_aes_256_ocb(), NULL, NULL, NULL) != 1) goto cleanup;
	if (EVP_CIPHER_CTX_ctrl(ctx, EVP_CTRL_AEAD_SET_IVLEN, AEAD_NONCE_SIZE, NULL) != 1) goto cleanup;
	if (EVP_CIPHER_CTX_ctrl(ctx, EVP_CTRL_AEAD_SET_TAG, AEAD_TAG_SIZE, NULL) != 1) goto cleanup;
	if (EVP_EncryptInit_ex(ctx, NULL, NULL, key, nonce) != 1) goto cleanup;

	int totalLen;
	int len;
	if (EVP_EncryptUpdate(ctx, NULL, &len, associatedData, (int)adLen) != 1) goto cleanup;
	if (EVP_EncryptUpdate(ctx, ciphertext, &len, plaintext, (int)ptLen) != 1) goto cleanup;
	totalLen = len;
	if (EVP_EncryptFinal_ex(ctx, ciphertext + totalLen, &len) != 1) goto cleanup;
	totalLen += len;
	if (totalLen != (int)ptLen) goto cleanup;

	if (EVP_CIPHER_CTX_ctrl(ctx, EVP_CTRL_AEAD_GET_TAG, AEAD_TAG_SIZE, tag) != 1) goto cleanup;

	result = true;
cleanup:
	EVP_CIPHER_CTX_free(ctx);
	return result;
}

bool aead_decrypt(unsigned char* plaintext, const unsigned char* ciphertext, size_t ctLen, const unsigned char* key, const unsigned char* tag, const unsigned char* associatedData, size_t adLen, const unsigned char* nonce) {
	bool result = false;

	if (adLen > INT_MAX) return false;
	if (ctLen > INT_MAX) return false;

	unsigned char tagBuffer[AEAD_TAG_SIZE];
	memcpy(tagBuffer, tag, AEAD_TAG_SIZE);

	EVP_CIPHER_CTX* ctx = EVP_CIPHER_CTX_new();
	if (ctx == NULL) return result;

	if (EVP_DecryptInit_ex(ctx, EVP_aes_256_ocb(), NULL, NULL, NULL) != 1) goto cleanup;
	if (EVP_CIPHER_CTX_ctrl(ctx, EVP_CTRL_AEAD_SET_IVLEN, AEAD_NONCE_SIZE, NULL) != 1) goto cleanup;
	if (EVP_DecryptInit_ex(ctx, NULL, NULL, key, nonce) != 1) goto cleanup;

	int totalLen;
	int len;
	if (EVP_DecryptUpdate(ctx, NULL, &len, associatedData, (int)adLen) != 1) goto cleanup;
	if (EVP_DecryptUpdate(ctx, plaintext, &len, ciphertext, (int)ctLen) != 1) goto cleanup;
	totalLen = len;
	if (EVP_CIPHER_CTX_ctrl(ctx, EVP_CTRL_AEAD_SET_TAG, AEAD_TAG_SIZE, tagBuffer) != 1) goto cleanup;
	if (EVP_DecryptFinal_ex(ctx, plaintext + totalLen, &len) != 1) goto cleanup;
	totalLen += len;
	if (totalLen != (int)ctLen) goto cleanup;

	result = true;
cleanup:
	EVP_CIPHER_CTX_free(ctx);
	return result;
}
