#pragma once

// Cramer-Shoup cryptosystem over Ed25519

#include <stdbool.h>

#include "ecc.h"

typedef struct eccrsh_pubkey eccrsh_pubkey;
typedef struct eccrsh_seckey eccrsh_seckey;

#define ECCRSH_PK_SIZE 96
#define ECCRSH_SK_SIZE 160

// High-level interface requires this + plaintext length buffer for ciphertext
#define ECCRSH_CT_OVERHEAD 144

// Functions to allocate and free memory without structual details
eccrsh_pubkey* eccrsh_newPubKey(void);
eccrsh_seckey* eccrsh_newSecKey(void);
void eccrsh_freePubKey(eccrsh_pubkey*);
void eccrsh_freeSecKey(eccrsh_seckey*);

void eccrsh_genKeys(eccrsh_pubkey*, eccrsh_seckey*);

void eccrsh_packPubKey(unsigned char* buffer, const eccrsh_pubkey* pk);
void eccrsh_unpackPubKey(eccrsh_pubkey* pk, const unsigned char* buffer);
void eccrsh_packSecKey(unsigned char* buffer, const eccrsh_seckey* sk);
void eccrsh_unpackSecKey(eccrsh_seckey* sk, const unsigned char* buffer);

bool eccrsh_encrypt(const eccrsh_pubkey* pk,
	const unsigned char* plaintext, size_t ptLen, const unsigned char* associatedData, size_t adLen,
	unsigned char* ciphertext, size_t ctMax, size_t* ctUsed);

bool eccrsh_decrypt(const eccrsh_seckey* sk,
	const unsigned char* ciphertext, size_t ctLen, const unsigned char* associatedData, size_t adLen,
	unsigned char* plaintext, size_t ptMax, size_t* ptUsed);

// Low-level interfaces
typedef struct eccrsh_ciphertext eccrsh_ciphertext;
#define ECCRSH_CT_POINT_SIZE 128
bool eccrsh_encryptRaw(const eccrsh_pubkey* pk, const ge25519* p, eccrsh_ciphertext* ct, bignum256modm k, ge25519* cdAlpha);
bool eccrsh_decryptRaw(const eccrsh_seckey* sk, const eccrsh_ciphertext* ct, const unsigned char* vBuffer, const bignum256modm alpha, ge25519* p);
void eccrsh_packCiphertext(unsigned char* buffer, const eccrsh_ciphertext* ct);
void eccrsh_unpackCiphertext(eccrsh_ciphertext* ct, const unsigned char** vBuffer, const unsigned char* buffer);
bool eccrsh_computeAlpha(bignum256modm alpha, const ge25519* u1, const ge25519* u2, const ge25519* e);
