#pragma once

// ElGamal cryptosystem over Ed25519

#include <stdbool.h>

#include "ecc.h"

typedef struct ecelgamal_pubkey ecelgamal_pubkey;
typedef struct ecelgamal_seckey ecelgamal_seckey;

#define ECELGAMAL_PK_SIZE 32
#define ECELGAMAL_SK_SIZE 32

// High-level interface requires this + plaintext length buffer for ciphertext
#define ECELGAMAL_CT_OVERHEAD 80

// Functions to allocate and free memory without structual details
ecelgamal_pubkey* ecelgamal_newPubKey(void);
ecelgamal_seckey* ecelgamal_newSecKey(void);
void ecelgamal_freePubKey(ecelgamal_pubkey*);
void ecelgamal_freeSecKey(ecelgamal_seckey*);

void ecelgamal_genKeys(ecelgamal_pubkey*, ecelgamal_seckey*);
void ecelgamal_genPackedKeys(unsigned char* secBuffer, ecelgamal_pubkey* public, ecelgamal_seckey* secret);

void ecelgamal_packPubKey(unsigned char* buffer, const ecelgamal_pubkey* pk);
void ecelgamal_unpackPubKey(ecelgamal_pubkey* pk, const unsigned char* buffer);
void ecelgamal_packSecKey(unsigned char* buffer, const ecelgamal_seckey* sk);
void ecelgamal_unpackSecKey(ecelgamal_seckey* sk, const unsigned char* buffer);

// The nonce passed to these high-level interfaces must not repeat, but it need
// not be secret or random.

bool ecelgamal_encrypt(const ecelgamal_pubkey* pk,
	const unsigned char* plaintext, size_t ptLen, const unsigned char* associatedData, size_t adLen, const unsigned char* nonce,
	unsigned char* ciphertext, size_t ctMax, size_t* ctUsed);

bool ecelgamal_decrypt(const ecelgamal_seckey* sk,
	const unsigned char* ciphertext, size_t ctLen, const unsigned char* associatedData, size_t adLen, const unsigned char* nonce,
	unsigned char* plaintext, size_t ptMax, size_t* ptUsed);

// Low-level interfaces
typedef struct ecelgamal_ciphertext ecelgamal_ciphertext;
#define ECELGAMAL_CT_POINT_SIZE 64
bool ecelgamal_encryptRaw(const ecelgamal_pubkey* pk, const ge25519* p, ecelgamal_ciphertext* ct, bignum256modm y);
bool ecelgamal_decryptRaw(const ecelgamal_seckey* sk, const ecelgamal_ciphertext* ct, ge25519* p);
void ecelgamal_packCiphertext(unsigned char* buffer, const ecelgamal_ciphertext* ct);
void ecelgamal_unpackCiphertext(ecelgamal_ciphertext* ct, const unsigned char* buffer);
