#pragma once

// Competing AKEs, implemented with our primitives solely for performance
// comparison purposes.

#include <stdlib.h>
#include <stdbool.h>

#define ECDH_MSG_SIZE 32
#define ECDH_KEY_SIZE 32
#define ECDH_SECRETSTATE_SIZE 40

void ecdh_1_initiate_I(unsigned char* secretState, unsigned char* send);
bool ecdh_2_respond_R(const unsigned char* received, unsigned char* send, unsigned char* key);
bool ecdh_3_finish_I(const unsigned char* received, unsigned char* secretState, unsigned char* key);

#define THREEDH_PK_SIZE 32
#define THREEDH_SK_SIZE 32
#define THREEDH_PREKEY_SIZE 32
#define THREEDH_MSG_OVERHEAD 32
#define THREEDH_SECRETSTATE_SIZE 40
#define THREEDH_SCRATCH_KEYGEN_SIZE 40
#define THREEDH_KEY_SIZE 32

void threedh_genLongTermKeys(unsigned char* pk, unsigned char* sk, unsigned char* scratch);
void threedh_1_makePreKey_I(unsigned char* secretState, unsigned char* send);
bool threedh_2_exchange_R(const unsigned char* received,
	size_t idLen, const unsigned char* idR,
	const unsigned char* pkI,
	const unsigned char* skR,
	unsigned char* send, unsigned char* key);
void threedh_3_receive_I(const unsigned char* received, size_t idLen, unsigned char* idR);
bool threedh_3_exchange_I(const unsigned char* received, size_t idLen,
	const unsigned char* pkR,
	const unsigned char* skI,
	unsigned char* secretState,
	unsigned char* key);

#define XTHREEDH_PK_SIZE 32
#define XTHREEDH_SK_SIZE 32
#define XTHREEDH_SIGNED_PREKEY_SIZE 96
#define XTHREEDH_ONETIME_PREKEY_SIZE 32
#define XTHREEDH_MSG_OVERHEAD 32
#define XTHREEDH_SECRETSTATE_SIZE 40
#define XTHREEDH_KEY_SIZE 32

void xthreedh_genLongTermKeys(unsigned char* pk, unsigned char* sk);
void xthreedh_genSignedPrekey(const unsigned char* pk, const unsigned char* sk, unsigned char* secretSignedState, unsigned char* send);
void xthreedh_eraseSignedPrekey(unsigned char* secretSignedState);
void xthreedh_1_makeOneTimePreKey_I(unsigned char* secretOneTimeState, unsigned char* send);
bool xthreedh_2_verifySignedPrekey_R(const unsigned char* pkI, const unsigned char* signedPrekey);
bool xthreedh_2_exchange_R(
	const unsigned char* signedPrekey, const unsigned char* oneTimePrekey,
	size_t idLen, const unsigned char* idR,
	const unsigned char* pkI,
	const unsigned char* skR,
	unsigned char* send, unsigned char* key);
void xthreedh_3_receive_I(const unsigned char* received, size_t idLen, unsigned char* idR);
bool xthreedh_3_exchange_I(const unsigned char* received, size_t idLen,
	const unsigned char* pkR,
	const unsigned char* skI,
	const unsigned char* secretSignedState,
	unsigned char* secretOneTimeState,
	unsigned char* key);

#define SIGMAR_PK_SIZE 32
#define SIGMAR_SK_SIZE 32
#define SIGMAR_EX1_MSG_SIZE 32
#define SIGMAR_EX2_MSG_OVERHEAD 96
#define SIGMAR_SECRETSTATE_I_SIZE 168
#define SIGMAR_SECRETSTATE_R_SIZE 104
#define SIGMAR_KEY_SIZE 32

void sigmar_genLongTermKeys(unsigned char* pk, unsigned char* sk);
void sigmar_1_initiateI(unsigned char* secretState, unsigned char* send);
void sigmar_2_respond_R(const unsigned char* received, unsigned char* secretState, unsigned char* send);
bool sigmar_3_respond_I(const unsigned char* received,
	size_t idLen, const unsigned char* idI,
	const unsigned char* pkI,
	const unsigned char* skI,
	unsigned char* secretState,
	unsigned char* send);
void sigmar_4_receive_R(const unsigned char* received, size_t idLen, unsigned char* idI);
bool sigmar_4_complete_R(const unsigned char* received,
	size_t idLen, const unsigned char* idI, const unsigned char* idR,
	const unsigned char* pkI, const unsigned char* pkR,
	const unsigned char* skR,
	unsigned char* secretState,
	unsigned char* send, unsigned char* key);
void sigmar_5_receive_I(const unsigned char* received, size_t idLen, unsigned char* idR);
bool sigmar_5_complete_I(const unsigned char* received,
	size_t idLen, const unsigned char* idR,
	const unsigned char* pkR,
	unsigned char* secretState,
	unsigned char* key);

