#include <stdlib.h>
#include <sys/mman.h>
#include <unistd.h>

#include "secmem.h"
#include "crypto.h"

size_t securePageSize(void) {
	long size = sysconf(_SC_PAGE_SIZE);
	if (size <= 0) return 0;
	return (size_t)size;
}

void* mallocSecurePage(size_t pages) {
	size_t pageSize = securePageSize();
	size_t length = pages * pageSize;
	void* ptr = mmap(NULL, length, PROT_READ|PROT_WRITE, MAP_SHARED|MAP_ANONYMOUS|MAP_LOCKED, -1, 0);
	if (ptr == MAP_FAILED) return NULL;
	if (mlock(ptr, length) != 0) {
		munmap(ptr, length);
		return NULL;
	}
	return ptr;
}

void secureErase(void* ptr, size_t size) {
	volatile char* p = ptr;
	for (size_t i = 0; i < size; ++i) {
		*p++ = 0;
	}
}

void freeSecurePage(void* ptr) {
	size_t pageSize = securePageSize();
	munlock(ptr, pageSize);
	munmap(ptr, pageSize);
}
